@@ -17,6 +17,8 @@ use crate::entities::EntityMeta;
 use crate::{Component, Entity, World};
 
 /// A collection of component types to fetch from a [`World`](crate::World)
+///
+/// The interface of this trait is a private implementation detail.
 pub trait Query {
     #[doc(hidden)]
     type Fetch: for<'a> Fetch<'a>;
@@ -1042,7 +1044,7 @@ smaller_tuples_too!(tuple_impl, O, N, M, L, K, J, I, H, G, F, E, D, C, B, A);
 
 /// A prepared query can be stored independently of the [`World`] to amortize query set-up costs.
 pub struct PreparedQuery<Q: Query> {
-    memo: (u64, u64),
+    memo: (u64, u32),
     state: Box<[(usize, <Q::Fetch as Fetch<'static>>::State)]>,
     fetch: Box<[Option<Q::Fetch>]>,
 }
@@ -1306,6 +1308,15 @@ impl<'q, Q: Query> View<'q, Q> {
     ///
     /// Will yield `None` if the entity does not exist or does not match the query.
     pub fn get_mut<'m>(&'m mut self, entity: Entity) -> Option<QueryItem<'q, Q>> {
+        unsafe { self.get_unchecked(entity) }
+    }
+
+    /// Like `get_mut`, but allows simultaneous access to multiple entities
+    ///
+    /// # Safety
+    ///
+    /// Must not be invoked while any unique borrow of the fetched components of `entity` is live.
+    pub unsafe fn get_unchecked(&self, entity: Entity) -> Option<QueryItem<'q, Q>> {
         let meta = self.meta.get(entity.id as usize)?;
         if meta.generation != entity.generation {
             return None;
@@ -1313,7 +1324,46 @@ impl<'q, Q: Query> View<'q, Q> {
 
         self.fetch[meta.location.archetype as usize]
             .as_ref()
-            .map(|fetch| unsafe { fetch.get(meta.location.index as usize) })
+            .map(|fetch| fetch.get(meta.location.index as usize))
+    }
+
+    /// Like `get_mut`, but allows checked simultaneous access to multiple entities
+    ///
+    /// For N > 3, the check for distinct entities will clone the array and take O(N log N) time.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use hecs::World;
+    /// let mut world = World::new();
+    ///
+    /// let a = world.spawn((1, 1.0));
+    /// let b = world.spawn((2, 4.0));
+    /// let c = world.spawn((3, 9.0));
+    ///
+    /// let mut query = world.query_mut::<&mut i32>();
+    /// let mut view = query.view();
+    /// let [a,b,c] = view.get_mut_n([a, b, c]);
+    ///
+    /// assert_eq!(*a.unwrap(), 1);
+    /// assert_eq!(*b.unwrap(), 2);
+    /// assert_eq!(*c.unwrap(), 3);
+    /// ```
+    pub fn get_mut_n<const N: usize>(
+        &mut self,
+        entities: [Entity; N],
+    ) -> [Option<QueryItem<'q, Q>>; N] {
+        assert_distinct(&entities);
+
+        let mut items = [(); N].map(|()| None);
+
+        for (item, entity) in items.iter_mut().zip(entities) {
+            unsafe {
+                *item = self.get_unchecked(entity);
+            }
+        }
+
+        items
     }
 }
 
@@ -1367,6 +1417,15 @@ impl<'q, Q: Query> PreparedView<'q, Q> {
     ///
     /// Will yield `None` if the entity does not exist or does not match the query.
     pub fn get_mut<'m>(&'m mut self, entity: Entity) -> Option<QueryItem<'q, Q>> {
+        unsafe { self.get_unchecked(entity) }
+    }
+
+    /// Like `get_mut`, but allows simultaneous access to multiple entities
+    ///
+    /// # Safety
+    ///
+    /// Must not be invoked while any unique borrow of the fetched components of `entity` is live.
+    pub unsafe fn get_unchecked(&self, entity: Entity) -> Option<QueryItem<'q, Q>> {
         let meta = self.meta.get(entity.id as usize)?;
         if meta.generation != entity.generation {
             return None;
@@ -1374,10 +1433,48 @@ impl<'q, Q: Query> PreparedView<'q, Q> {
 
         self.fetch[meta.location.archetype as usize]
             .as_ref()
-            .map(|fetch| unsafe { fetch.get(meta.location.index as usize) })
+            .map(|fetch| fetch.get(meta.location.index as usize))
+    }
+
+    /// Like `get_mut`, but allows checked simultaneous access to multiple entities
+    ///
+    /// See [`View::get_mut_n`] for details.
+    pub fn get_mut_n<const N: usize>(
+        &mut self,
+        entities: [Entity; N],
+    ) -> [Option<QueryItem<'q, Q>>; N] {
+        assert_distinct(&entities);
+
+        let mut items = [(); N].map(|()| None);
+
+        for (item, entity) in items.iter_mut().zip(entities) {
+            unsafe {
+                *item = self.get_unchecked(entity);
+            }
+        }
+
+        items
     }
 }
 
+fn assert_distinct<const N: usize>(entities: &[Entity; N]) {
+    match N {
+        1 => (),
+        2 => assert_ne!(entities[0], entities[1]),
+        3 => {
+            assert_ne!(entities[0], entities[1]);
+            assert_ne!(entities[1], entities[2]);
+            assert_ne!(entities[2], entities[0]);
+        }
+        _ => {
+            let mut entities = *entities;
+            entities.sort_unstable();
+            for index in 0..N - 1 {
+                assert_ne!(entities[index], entities[index + 1]);
+            }
+        }
+    }
+}
 #[cfg(test)]
 mod tests {
     use super::*;
