@@ -909,7 +909,7 @@ mod tests {
             Token::TupleEnd,
             Token::Tuple { len: 1 },
             Token::Tuple { len: 1 },
-            Token::U64(e2.to_bits()),
+            Token::U64(e2.to_bits().into()),
             Token::TupleEnd,
             Token::TupleEnd,
             Token::TupleEnd,
@@ -923,7 +923,7 @@ mod tests {
             Token::TupleEnd,
             Token::Tuple { len: 3 },
             Token::Tuple { len: 1 },
-            Token::U64(e0.to_bits()),
+            Token::U64(e0.to_bits().into()),
             Token::TupleEnd,
             Token::Tuple { len: 1 },
             Token::NewtypeStruct { name: "Position" },
@@ -952,7 +952,7 @@ mod tests {
             Token::TupleEnd,
             Token::Tuple { len: 2 },
             Token::Tuple { len: 1 },
-            Token::U64(e1.to_bits()),
+            Token::U64(e1.to_bits().into()),
             Token::TupleEnd,
             Token::Tuple { len: 1 },
             Token::NewtypeStruct { name: "Position" },
