@@ -18,7 +18,7 @@ use core::{fmt, ptr};
 #[cfg(feature = "std")]
 use std::error::Error;
 
-use hashbrown::{HashMap, HashSet};
+use hashbrown::hash_map::{Entry, HashMap};
 
 use crate::alloc::boxed::Box;
 use crate::archetype::{Archetype, TypeIdMap, TypeInfo};
@@ -327,7 +327,9 @@ impl World {
             .bundle_to_archetype
             .entry(TypeId::of::<T>())
             .or_insert_with(|| {
-                T::with_static_ids(|ids| archetypes.get(ids, &|| T::static_type_info()))
+                T::with_static_ids(|ids| {
+                    archetypes.get(ids, &|| T::with_static_type_info(|info| info.to_vec()))
+                })
             });
 
         self.archetypes.archetypes[archetype_id as usize].reserve(additional);
@@ -404,8 +406,8 @@ impl World {
         QueryMut::new(&self.entities.meta, &mut self.archetypes.archetypes)
     }
 
-    pub(crate) fn memo(&self) -> (u64, u64) {
-        (self.id, self.archetypes.generation)
+    pub(crate) fn memo(&self) -> (u64, u32) {
+        (self.id, self.archetypes.generation())
     }
 
     pub(crate) fn entities_meta(&self) -> &[EntityMeta] {
@@ -576,11 +578,11 @@ impl World {
                 target_storage = self.archetypes.get_insert_target(graph_origin, &components);
                 &target_storage
             }
-            Some(key) => match self.insert_edges.get(&(graph_origin, key)) {
-                Some(x) => x,
-                None => {
-                    let t = self.archetypes.get_insert_target(graph_origin, &components);
-                    self.insert_edges.entry((graph_origin, key)).or_insert(t)
+            Some(key) => match self.insert_edges.entry((graph_origin, key)) {
+                Entry::Occupied(entry) => entry.into_mut(),
+                Entry::Vacant(entry) => {
+                    let target = self.archetypes.get_insert_target(graph_origin, &components);
+                    entry.insert(target)
                 }
             },
         };
@@ -715,22 +717,20 @@ impl World {
         remove_edges: &mut IndexTypeIdMap<u32>,
         old_archetype: u32,
     ) -> u32 {
-        match remove_edges.get(&(old_archetype, TypeId::of::<T>())) {
-            Some(&x) => x,
-            None => {
-                let removed = T::with_static_ids(|ids| ids.iter().copied().collect::<HashSet<_>>());
-                let info = archetypes.archetypes[old_archetype as usize]
-                    .types()
-                    .iter()
-                    .cloned()
-                    .filter(|x| !removed.contains(&x.id()))
-                    .collect::<Vec<_>>();
+        match remove_edges.entry((old_archetype, TypeId::of::<T>())) {
+            Entry::Occupied(entry) => *entry.into_mut(),
+            Entry::Vacant(entry) => {
+                let info = T::with_static_type_info(|removed| {
+                    archetypes.archetypes[old_archetype as usize]
+                        .types()
+                        .iter()
+                        .filter(|x| removed.binary_search(x).is_err())
+                        .cloned()
+                        .collect::<Vec<_>>()
+                });
                 let elements = info.iter().map(|x| x.id()).collect::<Box<_>>();
                 let index = archetypes.get(&*elements, move || info);
-
-                remove_edges.insert((old_archetype, TypeId::of::<T>()), index);
-
-                index
+                *entry.insert(index)
             }
         }
     }
@@ -897,7 +897,7 @@ impl World {
     /// assert_ne!(initial_gen, world.archetypes_generation());
     /// ```
     pub fn archetypes_generation(&self) -> ArchetypesGeneration {
-        ArchetypesGeneration(self.archetypes.generation)
+        ArchetypesGeneration(self.archetypes.generation())
     }
 
     /// Number of currently live entities
@@ -1094,7 +1094,7 @@ impl<A: DynamicBundle> core::iter::FromIterator<A> for World {
 
 /// Determines freshness of information derived from [`World::archetypes`]
 #[derive(Debug, Copy, Clone, Eq, PartialEq)]
-pub struct ArchetypesGeneration(u64);
+pub struct ArchetypesGeneration(u32);
 
 /// Entity IDs created by [`World::spawn_batch`]
 pub struct SpawnBatchIter<'a, I>
@@ -1194,7 +1194,6 @@ struct ArchetypeSet {
     /// Maps sorted component type sets to archetypes
     index: HashMap<Box<[TypeId]>, u32>,
     archetypes: Vec<Archetype>,
-    generation: u64,
 }
 
 impl ArchetypeSet {
@@ -1203,7 +1202,6 @@ impl ArchetypeSet {
         Self {
             index: Some((Box::default(), 0)).into_iter().collect(),
             archetypes: vec![Archetype::new(Vec::new())],
-            generation: 0,
         }
     }
 
@@ -1224,14 +1222,11 @@ impl ArchetypeSet {
         self.archetypes.push(Archetype::new(info));
         let old = self.index.insert(components, x);
         debug_assert!(old.is_none(), "inserted duplicate archetype");
-        self.post_insert();
         x
     }
 
     /// Returns archetype ID and starting location index
     fn insert_batch(&mut self, archetype: Archetype) -> (u32, u32) {
-        use hashbrown::hash_map::Entry;
-
         let ids = archetype
             .types()
             .iter()
@@ -1253,14 +1248,13 @@ impl ArchetypeSet {
                 let id = self.archetypes.len() as u32;
                 self.archetypes.push(archetype);
                 x.insert(id);
-                self.post_insert();
                 (id, 0)
             }
         }
     }
 
-    fn post_insert(&mut self) {
-        self.generation += 1;
+    fn generation(&self) -> u32 {
+        self.archetypes.len() as u32
     }
 
     fn get_insert_target(&mut self, src: u32, components: &impl DynamicBundle) -> InsertTarget {
